﻿using ModuleTools.Extensions;
using System;
using System.Windows.Forms;

namespace DesktopTools.BaseControls
{
	/// <summary>
	/// Описание контрола для работы со значением свойства класса
	/// </summary>
	public abstract partial class AbstractBaseControl : UserControl
	{
		/// <summary>
		/// Оригинальное значение (требуется при откате изменений)
		/// </summary>
		protected object _originalValue;

		/// <summary>
		/// Необходимость проверки при получении данных
		/// </summary>
		protected bool _mustFilling;

		/// <summary>
		/// Название свойства, по которму идет отображение
		/// </summary>
		protected string _propertyName;

		#region Событие изменения значения
		/// <summary>
		/// Событие изменения значения в контроле
		/// </summary>
		private event Action OnValueChange;

		/// <summary>
		/// Событие изменения значения в контроле
		/// </summary>
		public event Action OnValueChangeEvent { add { OnValueChange += value; } remove { OnValueChange -= value; } }

		/// <summary>
		/// Вызов события в дочерних контролах
		/// </summary>
		protected void CallOnValueChangeEvent() => OnValueChange?.Invoke();
		#endregion

		/// <summary>
		/// Конструктор
		/// </summary>
		/// <param name="propertyName">Название свойства</param
		/// <param name="mustFilling">Должно ли оно быть заполненным обязательно</param>
		/// <param name="readOnly">Устанвока контрола в режим только просмотра</param>
		public AbstractBaseControl(string propertyName, bool mustFilling, bool readOnly)
		{
			InitializeComponent();
			_propertyName = propertyName;
			_mustFilling = mustFilling;
			panelControl.Enabled = !readOnly;
			SetDefaultValue();
		}

		/// <summary>
		/// Установка _originalValue в значение по умолчанию
		/// </summary>
		protected abstract void SetDefaultValue(); 

		#region Работа с заголовком
		/// <summary>
		/// Установка заголовка
		/// </summary>
		/// <param name="title"></param>
		/// <returns>Ширину полученного загаловка</returns>
		public int SetTitle(string title)
		{
			labelTitle.Text = title;
			labelTitle.AutoSize = true;
			return labelTitle.Width;
		}

		/// <summary>
		/// Установка ширины заголовка (вызывается через событие, чтобы у всех заголовков получилась одна ширина)
		/// </summary>
		/// <param name="width"></param>
		public void SetTitleWidth(int width)
		{
			labelTitle.AutoSize = false;
			labelTitle.Width = width;
		}
		#endregion

		#region Работа со значением
		/// <summary>
		/// Извлечение значения из свойства объекта
		/// </summary>
		/// <param name="obj"></param>
		public void SetValue(object obj)
		{
			if (obj != null && _propertyName.IsNotEmpty())
			{
				var property = obj.GetType().GetProperty(_propertyName);
				if (property != null)
				{
					_originalValue = property.GetValue(obj);
					SetValueToControl(_originalValue);
				}
			}
		}

		/// <summary>
		/// Установка значения в контрол
		/// </summary>
		/// <param name="value"></param>
		protected abstract void SetValueToControl(object value);

		/// <summary>
		/// Сброс значения
		/// </summary>
		public abstract void DropValue();

		/// <summary>
		/// Проверка на заполненность
		/// </summary>
		/// <returns></returns>
		public abstract bool CheckValue();

		/// <summary>
		/// Получение значение с контрола
		/// </summary>
		/// <returns></returns>
		protected abstract object GetValueFromControl();

		/// <summary>
		/// Заполнение свойства объекта значением из контрола 
		/// </summary>
		/// <param name="obj"></param>
		public void GetValue(object obj)
		{
			if (obj != null && _propertyName.IsNotEmpty())
			{
				var property = obj.GetType().GetProperty(_propertyName);
				if (property != null)
				{
					property.SetValue(obj, GetValueFromControl());
				}
			}
		}
		#endregion
	}
}
